@@ -9,7 +9,7 @@ use warnings;
 
 use Type::Library
    -base,
-   -declare => qw/ ConstArrayRef ConstHashRef /;
+   -declare => qw/ Const /;
 
 use Const::Fast ();
 use List::Util 1.33 ();
@@ -24,36 +24,20 @@ use Types::TypeTiny ();
 
 use namespace::autoclean 0.28;
 
-our $VERSION = 'v0.2.1';
+our $VERSION = 'v0.3.0';
 
 
-declare ConstArrayRef,
-  as ArrayRef,
+declare Const,
+  as Ref,
   where   \&__is_readonly,
   message {
-    return ArrayRef->get_message($_) unless ArrayRef->check($_);
     return "$_ is not readonly";
   },
-  constraint_generator => \&__arrayref_constraint_generator,
+  constraint_generator => \&__constraint_generator,
   coercion_generator   => \&__coercion_generator;
 
-coerce ConstArrayRef,
-  from ArrayRef,
-  via \&__coerce_constant;
-
-
-declare ConstHashRef,
-  as HashRef,
-  where   \&__is_readonly,
-  message {
-    return HashRef->get_message($_) unless HashRef->check($_);
-    return "$_ is not readonly";
-  },
-  constraint_generator => \&__hashref_constraint_generator,
-  coercion_generator   => \&__coercion_generator;
-
-coerce ConstHashRef,
-  from HashRef,
+coerce Const,
+  from Ref,
   via \&__coerce_constant;
 
 sub __coerce_constant {
@@ -62,39 +46,6 @@ sub __coerce_constant {
     return $value;
 }
 
-sub __arrayref_constraint_generator {
-    return ConstArrayRef unless @_;
-
-    my $param = shift;
-    Types::TypeTiny::TypeTiny->check($param)
-        or _croak("Parameter to ConstArrayRef[`a] expected to be a type constraint; got $param");
-
-    _croak("Only one parameter to ConstArrayRef[`a] expected; got @{[ 1 + @_ ]}.")
-        if @_;
-
-    my $psub = ArrayRef->parameterize($param)->constraint;
-
-    return sub {
-        return $psub->($_) && __is_readonly($_);
-    };
-
-}
-
-sub __hashref_constraint_generator {
-    return ConstHashRef unless @_;
-
-    my $param = shift;
-    Types::TypeTiny::TypeTiny->check($param)
-        or _croak("Parameter to ConstHashRef[`a] expected to be a type constraint; got $param");
-
-    my $psub = HashRef->parameterize($param)->constraint;
-
-    return sub {
-        return psub->($_) && __is_readonly($_);
-    };
-
-}
-
 sub __is_readonly {
     if ( is_ArrayRef($_) ) {
         return Internals::SvREADONLY(@$_)
@@ -113,6 +64,23 @@ sub __is_readonly {
     }
 }
 
+sub __constraint_generator {
+    return Const unless @_;
+
+    my $param = shift;
+    Types::TypeTiny::TypeTiny->check($param)
+        or _croak("Parameter to Const[`a] expected to be a type constraint; got $param");
+
+    _croak("Only one parameter to Const[`a] expected; got @{[ 1 + @_ ]}.")
+        if @_;
+
+    my $psub = $param->constraint;
+
+    return sub {
+        return $psub->($_) && __is_readonly($_);
+    };
+}
+
 sub __coercion_generator {
     my ( $parent, $child, $param ) = @_;
 
@@ -151,7 +119,7 @@ Types::Const - Types that coerce references to read-only
 
 =head1 VERSION
 
-version v0.2.1
+version v0.3.0
 
 =head1 SYNOPSIS
 
@@ -169,18 +137,25 @@ version v0.2.1
 
 =head1 DESCRIPTION
 
-The type library provides types that force read-only hash and array
-reference attributes to be deeply read-only.
+This is an I<experimental> type library that provides types that force
+read-only hash and array reference attributes to be deeply read-only.
+
+See the L<known issues|/KNOWN_ISSUES> below for a discussion of
+side-effects.
 
 =head1 TYPES
 
-=head2 C<ConstArrayRef[`a]>
+=head2 C<Const[`a]>
 
-A read-only array reference.
+Any defined reference value that is read-only.
 
-=head2 C<ConstHashRef[`a]>
+If parameterized, then the referred value must also pass the type
+constraint, for example C<Const[HashRef[Int]]> must a a hash reference
+with integer values.
 
-A read-only hash reference.
+It supports coercions to read-only.
+
+This was added in v0.3.0.
 
 =head1 SEE ALSO
 
@@ -190,12 +165,18 @@ L<Type::Tiny>
 
 L<Types::Standard>
 
-=head1 SOURCE
+=head1 KNOWN ISSUES
 
-The development version is on github at L<https://github.com/robrwo/Types-Const>
-and may be cloned from L<git://github.com/robrwo/Types-Const.git>
+=head2 Side-effects of read-only data structures
 
-=head1 BUGS
+A side-effect of read-only data structures is that an exception will
+be thrown if you attempt to fetch the value of a non-existent key:
+
+    Attempt to access disallowed key 'foo' in a restricted hash
+
+The work around for this is to check that a key exists beforehand.
+
+=head2 Bug reports and feature requests
 
 Please report any bugs or feature requests on the bugtracker website
 L<https://github.com/robrwo/Types-Const/issues>
@@ -204,6 +185,11 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SOURCE
+
+The development version is on github at L<https://github.com/robrwo/Types-Const>
+and may be cloned from L<git://github.com/robrwo/Types-Const.git>
+
 =head1 AUTHOR
 
 Robert Rothenberg <rrwo@cpan.org>
